"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.inferDetailsFromExisting = exports.prepare = void 0;
const clc = require("cli-color");
const ensureCloudBuildEnabled_1 = require("./ensureCloudBuildEnabled");
const functionsDeployHelper_1 = require("./functionsDeployHelper");
const utils_1 = require("../../utils");
const prepareFunctionsUpload_1 = require("./prepareFunctionsUpload");
const prompts_1 = require("./prompts");
const backend = require("./backend");
const ensureApiEnabled = require("../../ensureApiEnabled");
const functionsConfig = require("../../functionsConfig");
const functionsEnv = require("../../functions/env");
const previews_1 = require("../../previews");
const projectUtils_1 = require("../../projectUtils");
const track_1 = require("../../track");
const runtimes = require("./runtimes");
const validate = require("./validate");
const utils = require("../../utils");
const logger_1 = require("../../logger");
const triggerRegionHelper_1 = require("./triggerRegionHelper");
const checkIam_1 = require("./checkIam");
function hasUserConfig(config) {
    return Object.keys(config).length > 1;
}
function hasDotenv(opts) {
    return previews_1.previews.dotenv && functionsEnv.hasUserEnvs(opts);
}
async function maybeEnableAR(projectId) {
    if (previews_1.previews.artifactregistry) {
        return ensureApiEnabled.check(projectId, "artifactregistry.googleapis.com", "functions", true);
    }
    await ensureApiEnabled.ensure(projectId, "artifactregistry.googleapis.com", "functions");
    return true;
}
async function prepare(context, options, payload) {
    if (!options.config.src.functions) {
        return;
    }
    const runtimeDelegate = await runtimes.getRuntimeDelegate(context, options);
    logger_1.logger.debug(`Validating ${runtimeDelegate.name} source`);
    await runtimeDelegate.validate();
    logger_1.logger.debug(`Building ${runtimeDelegate.name} source`);
    await runtimeDelegate.build();
    const projectId = projectUtils_1.needProjectId(options);
    const checkAPIsEnabled = await Promise.all([
        ensureApiEnabled.ensure(projectId, "cloudfunctions.googleapis.com", "functions"),
        ensureApiEnabled.check(projectId, "runtimeconfig.googleapis.com", "runtimeconfig", true),
        ensureCloudBuildEnabled_1.ensureCloudBuildEnabled(projectId),
        maybeEnableAR(projectId),
    ]);
    context.runtimeConfigEnabled = checkAPIsEnabled[1];
    context.artifactRegistryEnabled = checkAPIsEnabled[3];
    const firebaseConfig = await functionsConfig.getFirebaseConfig(options);
    context.firebaseConfig = firebaseConfig;
    const runtimeConfig = await prepareFunctionsUpload_1.getFunctionsConfig(context);
    utils.assertDefined(options.config.src.functions.source, "Error: 'functions.source' is not defined");
    const source = options.config.src.functions.source;
    const firebaseEnvs = functionsEnv.loadFirebaseEnvs(firebaseConfig, projectId);
    const userEnvOpt = {
        functionsSource: options.config.path(source),
        projectId: projectId,
        projectAlias: options.projectAlias,
    };
    const userEnvs = functionsEnv.loadUserEnvs(userEnvOpt);
    const usedDotenv = hasDotenv(userEnvOpt);
    const tag = hasUserConfig(runtimeConfig)
        ? usedDotenv
            ? "mixed"
            : "runtime_config"
        : usedDotenv
            ? "dotenv"
            : "none";
    await track_1.track("functions_codebase_deploy_env_method", tag);
    logger_1.logger.debug(`Analyzing ${runtimeDelegate.name} backend spec`);
    const wantBackend = await runtimeDelegate.discoverSpec(runtimeConfig, firebaseEnvs);
    wantBackend.environmentVariables = Object.assign(Object.assign({}, userEnvs), firebaseEnvs);
    payload.functions = { backend: wantBackend };
    if (backend.someEndpoint(wantBackend, (e) => e.platform === "gcfv2")) {
        const V2_APIS = [
            "artifactregistry.googleapis.com",
            "run.googleapis.com",
            "eventarc.googleapis.com",
            "pubsub.googleapis.com",
            "storage.googleapis.com",
        ];
        const enablements = V2_APIS.map((api) => {
            return ensureApiEnabled.ensure(context.projectId, api, "functions");
        });
        await Promise.all(enablements);
    }
    if (backend.someEndpoint(wantBackend, () => true)) {
        utils_1.logBullet(clc.cyan.bold("functions:") +
            " preparing " +
            clc.bold(options.config.src.functions.source) +
            " directory for uploading...");
    }
    if (backend.someEndpoint(wantBackend, (e) => e.platform === "gcfv1")) {
        context.functionsSourceV1 = await prepareFunctionsUpload_1.prepareFunctionsUpload(runtimeConfig, options);
    }
    if (backend.someEndpoint(wantBackend, (e) => e.platform === "gcfv2")) {
        context.functionsSourceV2 = await prepareFunctionsUpload_1.prepareFunctionsUpload(undefined, options);
    }
    for (const endpoint of backend.allEndpoints(wantBackend)) {
        endpoint.environmentVariables = wantBackend.environmentVariables;
    }
    await Promise.all(Object.values(wantBackend.requiredAPIs).map((api) => {
        return ensureApiEnabled.ensure(projectId, api, "functions", false);
    }));
    validate.functionIdsAreValid(backend.allEndpoints(wantBackend));
    context.filters = functionsDeployHelper_1.getFilterGroups(options);
    const matchingBackend = backend.matchingBackend(wantBackend, (endpoint) => {
        return functionsDeployHelper_1.functionMatchesAnyGroup(endpoint, context.filters);
    });
    const haveBackend = await backend.existingBackend(context);
    await checkIam_1.ensureServiceAgentRoles(projectId, wantBackend, haveBackend);
    inferDetailsFromExisting(wantBackend, haveBackend, usedDotenv);
    await triggerRegionHelper_1.ensureTriggerRegions(wantBackend);
    await prompts_1.promptForFailurePolicies(options, matchingBackend, haveBackend);
    await prompts_1.promptForMinInstances(options, matchingBackend, haveBackend);
    await backend.checkAvailability(context, wantBackend);
}
exports.prepare = prepare;
function inferDetailsFromExisting(want, have, usedDotenv) {
    var _a;
    for (const wantE of backend.allEndpoints(want)) {
        const haveE = (_a = have.endpoints[wantE.region]) === null || _a === void 0 ? void 0 : _a[wantE.id];
        if (!haveE) {
            continue;
        }
        if (!usedDotenv) {
            wantE.environmentVariables = Object.assign(Object.assign({}, haveE.environmentVariables), wantE.environmentVariables);
        }
        if (!wantE.availableMemoryMb && haveE.availableMemoryMb) {
            wantE.availableMemoryMb = haveE.availableMemoryMb;
        }
        maybeCopyTriggerRegion(wantE, haveE);
    }
}
exports.inferDetailsFromExisting = inferDetailsFromExisting;
function maybeCopyTriggerRegion(wantE, haveE) {
    if (!backend.isEventTriggered(wantE) || !backend.isEventTriggered(haveE)) {
        return;
    }
    if (wantE.eventTrigger.region || !haveE.eventTrigger.region) {
        return;
    }
    if (JSON.stringify(haveE.eventTrigger.eventFilters) !==
        JSON.stringify(wantE.eventTrigger.eventFilters)) {
        return;
    }
    wantE.eventTrigger.region = haveE.eventTrigger.region;
}
